/*
   (C) 2001 by Argonne National Laboratory.
       See COPYRIGHT in top-level directory.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tbb/tick_count.h"
#include "parallel2.h"

#ifdef _DIST_
    CnC::dist_cnc_init<parallel2_context>_dinit;
#endif

double h = 0.0;
double sum = 0.0;

int compute::execute(const int & t,  parallel2_context & c) const
{
   int i = t;
   double x = h * ((double)i - 0.5);
   double f = (4.0 / (1.0 + x*x));
   //sum = sum + (f * h);
   //printf("%f\n", sum);
   c.fvalues.put(t, f);
   return CnC::CNC_Success;
}

int main( int argc, char *argv[] )
{
    tbb::tick_count t0 = tbb::tick_count::now();

    int done = 0, n, i;
    double PI25DT = 3.141592653589793238462643;
    double mypi, pi, x, p;
    double sum;
    int np = 1;
    n = 0;

    parallel2_context c;
	  
    n = atoi(argv[1]);
    //np = atoi(argv[2]); 
    //CnC::debug::set_num_threads(np);

    h = 1.0 / (double) n;
	  
    for (i = 1; i <= n; i++){
      c.trapezoids.put(i);
    }
    c.wait();

    CnC::item_collection<int,double>::const_iterator cii;
    for (cii = c.fvalues.begin(); cii != c.fvalues.end(); cii++) {
      p = *(cii->second);
      sum += (p * h);
    }

    double error = fabs(sum - PI25DT); 	
    //c.wait();

    tbb::tick_count t1 = tbb::tick_count::now();

    printf("n       \t= %24d\n", n);	
    printf("our pi  \t= %24.16f\n", sum);
    printf("real pi \t= %24.16f\n", PI25DT);
    printf("error   \t= %24.16f\n", error);
    printf("wall clock time = %24g\n\n", (t1-t0).seconds());
    //printf("number of processes: %s\n", getenv("CNC_NUM_THREADS"));
    printf("number of distributed processors = %d\n", 
	   CnC::Internal::distributor::numProcs());
 
    return 0;
}

            
