#include "main.h"

int compute::execute(const double & t, parallel_context & c) const
{
  double a = t;
  double error;
  int iter;

  tbb::tick_count t2 = tbb::tick_count::now();
  
  //this is the serial code we are running the parameter study on
  //this line should be replaced with whatever other serial code
  //in this example, mesh size = 512, tolerance = 10^-6, 
  //maximum iterations = 99999, &error and &iter are returned and 
  //a is the random parameter that was added to the tag collection in main
  poisson(512, 1.0e-06, 99999, &error, &iter, a);
  
  tbb::tick_count t3 = tbb::tick_count::now();

  //put iteration and error values into item collections for future use
  c.errors.put(t, error);
  c.iterations.put(t, iter);

  //print running time for one poisson call as well as error and #iter
  printf("%.2f %14.6e %6d %10.2f\n", a, error, iter, (t3-t2).seconds());

  return CnC::CNC_Success;
}

int main(int argc, char *argv[])
{
  int i, j, n;
  double a;
  double placeHolder = 0.0;

  i = atoi(argv[1]);

  //set number of threads
  if (argc > 2){
  n = atoi(argv[2]);
  CnC::debug::set_num_threads(n);
  }

  //create the cnc context
  parallel_context c;
  
  //beging timing
  tbb::tick_count t0 = tbb::tick_count::now();

  //insert a values into loop
  for (j = 0; j < i; j ++){
    srand(j);
    a = 1000.0 * ((double)rand()/(double)RAND_MAX);
    //execute is run in parallel ASAP after this line
    c.pvalue.put(a);
  }
  
  c.wait();

  //end timing
  tbb::tick_count t1 = tbb::tick_count::now();
  
  //print out total program running time
  printf("%.1f %14.1f %6.1f %10.2f\n\n", placeHolder, placeHolder, placeHolder,
	 (t1-t0).seconds());

  return 0;
}
