//********************************************************************************
// Copyright (c) 2007-2010 Intel Corporation. All rights reserved.              **
//                                                                              **
// Redistribution and use in source and binary forms, with or without           **
// modification, are permitted provided that the following conditions are met:  **
//   * Redistributions of source code must retain the above copyright notice,   **
//     this list of conditions and the following disclaimer.                    **
//   * Redistributions in binary form must reproduce the above copyright        **
//     notice, this list of conditions and the following disclaimer in the      **
//     documentation and/or other materials provided with the distribution.     **
//   * Neither the name of Intel Corporation nor the names of its contributors  **
//     may be used to endorse or promote products derived from this software    **
//     without specific prior written permission.                               **
//                                                                              **
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  **
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    **
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   **
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE     **
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR          **
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF         **
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS     **
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN      **
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)      **
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF       **
// THE POSSIBILITY OF SUCH DAMAGE.                                              **
//********************************************************************************
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tbb/tick_count.h"
#include "FindPrimes.h"


// Compute if a number (which is the tag value) is prime.
// If the number is prime, put the number in Primes item collection.
int compute::execute(const int & t, FindPrimes_context & c ) const
{
    int number = t;
    int factor = 3;

    while ( number % factor )
    {
        factor += 2;
    }
    if (factor == number)  // number is prime
    {
        // Put the number in the primes tag collection.
        c.primes.put(t, number);
    }
    
    return CnC::CNC_Success;
}

int main(int argc, char* argv[])
{
    bool verbose = false;
    int np = 1;
    int n = 0;
    int number_of_primes = 0;

    if (argc == 3) 
    {
        n = atoi(argv[1]);
	np = atoi(argv[2]);
    }
    else if (argc == 4 && 0 == strcmp("-v", argv[1]))
    {
        n = atoi(argv[2]);
	np = atoi(argv[3]);
        verbose = true;
    }
    else
    {
        fprintf(stderr,"Usage: primes [-v] n np\n");
        return -1;
    }
    CnC::debug::set_num_threads(np);
    
    // Create an instance of the context class which defines the graph
    FindPrimes_context c;

    printf("Determining primes from 1-%d \n",n);

    tbb::tick_count t0 = tbb::tick_count::now();

    // Env puts odd numbers from 3..n to prescribe the compute step instances
    for (int number = 3; number <= n; number += 2)
    {
        c.oddNums.put(number);
    }
    
    // Wait for all steps to finish
    c.wait();

    tbb::tick_count t1 = tbb::tick_count::now();

    number_of_primes = (int)c.primes.size() + 1;
    printf("Found %d primes in %g seconds\n", number_of_primes, (t1-t0).seconds());

    if (verbose)
    {
        // Print the prime numbers
        printf("%ld\n", 2);
        CnC::item_collection<int,int>::const_iterator cii;
        for (cii = c.primes.begin(); cii != c.primes.end(); cii++) {
            int p = *(cii->second);
            printf("%ld\n", p);
        }
    }
}

